/*
 * Copyright 2016 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3, as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *   Charles Kerr <charles.kerr@canonical.com>
 */

#pragma once

#include <core/signal.h>

#include <memory>
#include <string>

/**
 * Simple interface that emits signals on USB device state changes
 */
class UsbMonitor
{
public:
    UsbMonitor();
    virtual ~UsbMonitor();
    virtual core::Signal<const std::string&>& on_usb_disconnected() =0;
};

/**
 * Simple GUDev wrapper that notifies on android_usb device state changes
 */
class GUDevUsbMonitor: public UsbMonitor
{
public:
    GUDevUsbMonitor();
    virtual ~GUDevUsbMonitor();
    core::Signal<const std::string&>& on_usb_disconnected() override;

protected:
    class Impl;
    std::unique_ptr<Impl> impl;
};

